@@ -44,9 +44,16 @@ __version__ = "1.7.0"
 
 import contextlib
 import ctypes
-from ctypes import c_size_t, c_wchar, c_wchar_p, get_errno, sizeof
+from ctypes import (
+    c_size_t,
+    c_wchar,
+    c_wchar_p,
+    get_errno,
+    sizeof,
+)
 import os
 import platform
+from shutil import which
 import subprocess
 import time
 import warnings
@@ -59,7 +66,7 @@ HAS_DISPLAY = os.getenv("DISPLAY", False)
 EXCEPT_MSG = """
     Pyperclip could not find a copy/paste mechanism for your system.
     For more information, please visit
-    https://pyperclip.readthedocs.io/en/latest/introduction.html#not-implemented-error
+    https://pyperclip.readthedocs.io/en/latest/#not-implemented-error
     """
 
 ENCODING = "utf-8"
@@ -104,12 +111,16 @@ def _stringifyText(text) -> str:
 def init_osx_pbcopy_clipboard():
     def copy_osx_pbcopy(text):
         text = _stringifyText(text)  # Converts non-str values to str.
-        p = subprocess.Popen(["pbcopy", "w"], stdin=subprocess.PIPE, close_fds=True)
-        p.communicate(input=text.encode(ENCODING))
+        with subprocess.Popen(
+            ["pbcopy", "w"], stdin=subprocess.PIPE, close_fds=True
+        ) as p:
+            p.communicate(input=text.encode(ENCODING))
 
     def paste_osx_pbcopy():
-        p = subprocess.Popen(["pbpaste", "r"], stdout=subprocess.PIPE, close_fds=True)
-        stdout, stderr = p.communicate()
+        with subprocess.Popen(
+            ["pbpaste", "r"], stdout=subprocess.PIPE, close_fds=True
+        ) as p:
+            stdout = p.communicate()[0]
         return stdout.decode(ENCODING)
 
     return copy_osx_pbcopy, paste_osx_pbcopy
@@ -172,22 +183,22 @@ def init_xclip_clipboard():
         selection = DEFAULT_SELECTION
         if primary:
             selection = PRIMARY_SELECTION
-        p = subprocess.Popen(
+        with subprocess.Popen(
             ["xclip", "-selection", selection], stdin=subprocess.PIPE, close_fds=True
-        )
-        p.communicate(input=text.encode(ENCODING))
+        ) as p:
+            p.communicate(input=text.encode(ENCODING))
 
     def paste_xclip(primary=False):
         selection = DEFAULT_SELECTION
         if primary:
             selection = PRIMARY_SELECTION
-        p = subprocess.Popen(
+        with subprocess.Popen(
             ["xclip", "-selection", selection, "-o"],
             stdout=subprocess.PIPE,
             stderr=subprocess.PIPE,
             close_fds=True,
-        )
-        stdout, stderr = p.communicate()
+        ) as p:
+            stdout = p.communicate()[0]
         # Intentionally ignore extraneous output on stderr when clipboard is empty
         return stdout.decode(ENCODING)
 
@@ -203,19 +214,19 @@ def init_xsel_clipboard():
         selection_flag = DEFAULT_SELECTION
         if primary:
             selection_flag = PRIMARY_SELECTION
-        p = subprocess.Popen(
+        with subprocess.Popen(
             ["xsel", selection_flag, "-i"], stdin=subprocess.PIPE, close_fds=True
-        )
-        p.communicate(input=text.encode(ENCODING))
+        ) as p:
+            p.communicate(input=text.encode(ENCODING))
 
     def paste_xsel(primary=False):
         selection_flag = DEFAULT_SELECTION
         if primary:
             selection_flag = PRIMARY_SELECTION
-        p = subprocess.Popen(
+        with subprocess.Popen(
             ["xsel", selection_flag, "-o"], stdout=subprocess.PIPE, close_fds=True
-        )
-        stdout, stderr = p.communicate()
+        ) as p:
+            stdout = p.communicate()[0]
         return stdout.decode(ENCODING)
 
     return copy_xsel, paste_xsel
@@ -224,7 +235,7 @@ def init_xsel_clipboard():
 def init_klipper_clipboard():
     def copy_klipper(text):
         text = _stringifyText(text)  # Converts non-str values to str.
-        p = subprocess.Popen(
+        with subprocess.Popen(
             [
                 "qdbus",
                 "org.kde.klipper",
@@ -234,16 +245,16 @@ def init_klipper_clipboard():
             ],
             stdin=subprocess.PIPE,
             close_fds=True,
-        )
-        p.communicate(input=None)
+        ) as p:
+            p.communicate(input=None)
 
     def paste_klipper():
-        p = subprocess.Popen(
+        with subprocess.Popen(
             ["qdbus", "org.kde.klipper", "/klipper", "getClipboardContents"],
             stdout=subprocess.PIPE,
             close_fds=True,
-        )
-        stdout, stderr = p.communicate()
+        ) as p:
+            stdout = p.communicate()[0]
 
         # Workaround for https://bugs.kde.org/show_bug.cgi?id=342874
         # TODO: https://github.com/asweigart/pyperclip/issues/43
@@ -264,18 +275,18 @@ def init_dev_clipboard_clipboard():
         text = _stringifyText(text)  # Converts non-str values to str.
         if text == "":
             warnings.warn(
-                "Pyperclip cannot copy a blank string to the clipboard on Cygwin."
+                "Pyperclip cannot copy a blank string to the clipboard on Cygwin. "
                 "This is effectively a no-op."
             )
         if "\r" in text:
             warnings.warn("Pyperclip cannot handle \\r characters on Cygwin.")
 
-        with open("/dev/clipboard", "wt") as fo:
-            fo.write(text)
+        with open("/dev/clipboard", "wt") as fd:
+            fd.write(text)
 
     def paste_dev_clipboard() -> str:
-        with open("/dev/clipboard", "rt") as fo:
-            content = fo.read()
+        with open("/dev/clipboard") as fd:
+            content = fd.read()
         return content
 
     return copy_dev_clipboard, paste_dev_clipboard
@@ -311,17 +322,17 @@ def init_windows_clipboard():
     global HGLOBAL, LPVOID, DWORD, LPCSTR, INT
     global HWND, HINSTANCE, HMENU, BOOL, UINT, HANDLE
     from ctypes.wintypes import (
-        HGLOBAL,
-        LPVOID,
+        BOOL,
         DWORD,
-        LPCSTR,
-        INT,
-        HWND,
+        HANDLE,
+        HGLOBAL,
         HINSTANCE,
         HMENU,
-        BOOL,
+        HWND,
+        INT,
+        LPCSTR,
+        LPVOID,
         UINT,
-        HANDLE,
     )
 
     windll = ctypes.windll
@@ -476,17 +487,17 @@ def init_windows_clipboard():
 def init_wsl_clipboard():
     def copy_wsl(text):
         text = _stringifyText(text)  # Converts non-str values to str.
-        p = subprocess.Popen(["clip.exe"], stdin=subprocess.PIPE, close_fds=True)
-        p.communicate(input=text.encode(ENCODING))
+        with subprocess.Popen(["clip.exe"], stdin=subprocess.PIPE, close_fds=True) as p:
+            p.communicate(input=text.encode(ENCODING))
 
     def paste_wsl():
-        p = subprocess.Popen(
+        with subprocess.Popen(
             ["powershell.exe", "-command", "Get-Clipboard"],
             stdout=subprocess.PIPE,
             stderr=subprocess.PIPE,
             close_fds=True,
-        )
-        stdout, stderr = p.communicate()
+        ) as p:
+            stdout = p.communicate()[0]
         # WSL appends "\r\n" to the contents.
         return stdout[:-2].decode(ENCODING)
 
@@ -507,11 +518,11 @@ def determine_clipboard():
         "cygwin" in platform.system().lower()
     ):  # Cygwin has a variety of values returned by platform.system(),
         # such as 'CYGWIN_NT-6.1'
-        # FIXME: pyperclip currently does not support Cygwin,
+        # FIXME(pyperclip#55): pyperclip currently does not support Cygwin,
         # see https://github.com/asweigart/pyperclip/issues/55
         if os.path.exists("/dev/clipboard"):
             warnings.warn(
-                "Pyperclip's support for Cygwin is not perfect,"
+                "Pyperclip's support for Cygwin is not perfect, "
                 "see https://github.com/asweigart/pyperclip/issues/55"
             )
             return init_dev_clipboard_clipboard()
@@ -521,15 +532,14 @@ def determine_clipboard():
         return init_windows_clipboard()
 
     if platform.system() == "Linux":
-        with open("/proc/version", "r") as f:
-            if "Microsoft" in f.read():
-                return init_wsl_clipboard()
+        if which("wslconfig.exe"):
+            return init_wsl_clipboard()
 
     # Setup for the MAC OS X platform:
     if os.name == "mac" or platform.system() == "Darwin":
         try:
-            import Foundation  # check if pyobjc is installed
             import AppKit
+            import Foundation  # check if pyobjc is installed
         except ImportError:
             return init_osx_pbcopy_clipboard()
         else:
